<br /><br />

<div align="center">

<a href="[Speakeasy](https://speakeasyapi.dev/)">
  <img src="https://github.com/speakeasy-api/speakeasy/assets/68016351/e959f81a-b250-4003-8c5c-a45b9463fc95" alt="Speakeasy Logo" width="700">
<h2>Enterprise-Ready Developer Experience for your REST API</h2>
<p>Launch production-quality SDKs, Terraform providers, docs, and more in minutes :rocket:</p>
</a>
<br />
<br />

   <a href="https://speakeasyapi.dev/docs/create-client-sdks/"><img src="https://img.shields.io/static/v1?label=Docs&message=Quickstart&color=000&style=for-the-badge" /></a>
   <a href="https://join.slack.com/t/speakeasy-dev/shared_invite/zt-1cwb3flxz-lS5SyZxAsF_3NOq5xc8Cjw"><img src="https://img.shields.io/static/v1?label=Slack&message=Join&color=7289da&style=for-the-badge" /></a>

  ![Group 26](https://github.com/speakeasy-api/speakeasy/assets/68016351/159f0565-0cfa-468a-8fcd-7c3590eea355)

  ![ezgif com-video-to-gif (1)](https://github.com/speakeasy-api/speakeasy/assets/90289500/ff6972c4-4e8a-4a5c-a976-75e97cc42f5a)
</div>
<br />

## What is Speakeasy?

[Speakeasy](https://www.speakeasyapi.dev/) gives your users the developer experience that makes API integrations easy.

Don't put the burden of integration on your users. Take your APIs to market with best-in-class, enterprise-ready SDKs, Terraform providers, SDK docs, and more.

![Group 27](https://github.com/speakeasy-api/speakeasy/assets/68016351/07335321-2ae1-41de-a1b1-3be72df38a90)

### Key Features

* **LLM Powered API Maintenance** - Keep your API schema up to date and follow best practices with automatic maintenance powered by best-in-class LLMs. Try it out now in the CLI with `speakeasy suggest` or embed it into your GitHub workflow using our [action](https://github.com/speakeasy-api/sdk-generation-action). 
* **Production-Quality SDKs** - Ergonomic, type-safe, and fault-tolerant client libraries built for production use cases with idiomatics optimized for each language. Embed additional capabilities like auth, retries, pagination, and more using spec-based extensions. 
* **Production-Quality Terraform Providers** - Expose your API for infrastructure-as-code use cases with fully featured Terraform providers. 
* **SDK Docs** - Markdown usage snippets for every single operation and model in your SDK. Export usage snippets and host them on a branded SDK and API docs site using our Docusaurus React plugin. 
* **Distribution to Package Managers** - A complete pipeline for distributing your libraries to popular package managers like npm, PyPI, Maven, and Terraform Registry. 
* **Supports Open Standards** - Speakeasy works with your existing API framework by supporting OpenAPI3.X and other open standards. Plug and play not rip and replace. 

For upcoming features and bug fixes, check out our [public roadmap](https://github.com/orgs/speakeasy-api/projects/6/views/1).

## What is the Speakeasy CLI?

The [Speakeasy CLI](https://www.speakeasyapi.dev/docs/speakeasy-cli/getting-started) is a tool for interacting with the Speakeasy platform that brings the functionality of Speakeasy into your development workflow. 

You can run the Speakeasy CLI locally or in your CI/CD pipeline to validate your API specs, generate SDKs, and more. Looking to embed this into your GitHub CI/CD? Check out [these docs](https://speakeasyapi.dev/docs/create-client-sdks/github-setup/) instead. 

Here are a few key functions of the CLI:

* `generate` - Generate idiomatic client SDKs from your API specs.
* `validate` - Validate the correctness of your API specs. The CLI has a built-in command to validate your spec for SDK readiness and post helpful error messages.
* `suggest` - Use an LLM to autocorrect your spec validation failures. 
* `merge` - Work with your existing documentation workflows by merging your API specs into a single spec. 
* `auth` - Authenticate with the platform and manage API keys. 

For a complete list of commands check out our [reference](https://www.speakeasyapi.dev/docs/speakeasy-cli/getting-started) or type `speakeasy` and our interactive mode will take you through the available functions. 

## Design Choices

Each SDK generated by Speakeasy is idiomatic to the language it's generated for while sharing common patterns with SDKs for other languages to allow for switching between them. Using shared patterns also helps to keep the Speakeasy generation engine efficient while supporting many languages.

Here are some of the Speakeasy design decisions:

* Each SDK generally implements a base SDK class that contains the methods for each of the API endpoints defined in the OpenAPI document.
* Where possible, Speakeasy generates fully typed models from the OpenAPI document, separating models defined as components in the docs from models defined in line with operations.
* Based on the configuration in the OpenAPI document, Speakeasy annotates types using reflection metadata to determine how to serialize and deserialize them.
* To get SDKs to your end users quickly, Speakeasy generates full packages for each language that you should be able to publish to a package registry with little additional work. If you're interested in having a managed pipeline to your package manager, check out the Speakeasy Generation GitHub Action.

Want to learn more about the Speakeasy methodology? Take a look at [how the Speakeasy SDK generator compares to OSS options](https://www.speakeasyapi.dev/post/client-sdks-as-a-service). 

If you're interested in generating managed GitHub repos for your SDKs or enterprise support, [reach out to us](https://www.speakeasyapi.dev/request-access) or [schedule a chat](https://calendly.com/d/drw-t98-rpq/simon-sagar-speakeasy). We'd love to help you build out API DevEx.

If you're interested in having managed GitHub repos generated for your SDKs or enterprise support, [reach out to us](https://www.speakeasyapi.dev/request-access) or [schedule a chat](https://calendly.com/d/drw-t98-rpq/simon-sagar-speakeasy). We'd love to help you build out API DevEx.

> Speakeasy may capture metadata on usage of the Speakeasy CLI to better understand OpenAPI features so that we can build better code generators and other tools over time.

## Installation

### Homebrew (macOS and Linux)

```bash
brew install speakeasy-api/homebrew-tap/speakeasy
```

### Script (macOS and Linux)

```bash
curl -fsSL https://raw.githubusercontent.com/speakeasy-api/speakeasy/main/install.sh | sh
```

### Chocolatey (Windows)

```cmd
choco install speakeasy
```

### Manual Installation

Download the latest release for your platform from the [releases page](https://github.com/speakeasy-api/speakeasy/releases), extract, and add the binary to your path.

### Keeping Up to Date

The CLI will warn you if you're running an outdated version. To update the CLI run:

```bash
speakeasy update
```

Alternatively, install the latest version via your package manager.

## Getting Started With the Speakeasy CLI

Once you have installed the Speakeasy CLI, you can verify it's working by running:

```bash
speakeasy --help
```

See the Speakeasy docs for [how to get started with the Speakeasy CLI](https://www.speakeasyapi.dev/docs/speakeasy-cli/getting-started).

### Authenticating the Speakeasy CLI

The Speakeasy CLI depends on Speakeasy Platform APIs. Connect your Speakeasy CLI with the Speakeasy Platform by running:

```bash
speakeasy auth login
```
You'll be redirected to a login URL to select an existing workspace or create a new workspace on the platform. If your local network prevents you from accessing the login page prompted by the CLI, you can log in manually at [app.speakeasyapi.dev](https://app.speakeasyapi.dev), retrieve an API key, and populate a local environment variable named `SPEAKEASY_API_KEY` with the key.

<img width="1268" alt="Screenshot 2023-01-29 at 23 12 05" src="https://user-images.githubusercontent.com/68016351/215410983-b41dab8c-12b1-472c-a2fb-3325b881ff8e.png">

### SDK Generation

**Command**:

```bash
speakeasy generate sdk [flags]
```

**Options**:

```bash
  -b, --baseurl string   base URL for the API (only required if OpenAPI spec doesn't specify root server URLs)
  -d, --debug            enable writing debug files with broken code
  -h, --help             help for SDK
  -l, --lang string      language to generate SDK for, defaults to go (available options: go, python, typescript, java)
  -o, --out string       path to the output directory
  -s, --schema string    path to the OpenAPI schema
  -y, --auto-yes string  auto answer yes to all prompts
```

Take a look at the Speakeasy [getting started](https://www.speakeasyapi.dev/docs/speakeasy-cli/getting-started) page for in-depth information.

## Schema Validation

Note that you don't need to be logged in to the Speakeasy platform to validate your API schema.

**Command**:

```bash
speakeasy validate openapi [flags]
```

**Options**:

```bash
  -h, --help            help for OpenAPI
  -s, --schema string   path to the OpenAPI schema
```

## OpenAPI Usage

Note that you don't need to be logged in to the Speakeasy platform to view OpenAPI usage information.

**Command**:

Use the following command to return usage information for a given OpenAPI schema to a CSV. Usage information includes counts of OpenAPI features.

```bash
speakeasy usage [flags]
```

**Options**:

```bash
  -d, --debug         enable writing debug files with broken code
  -f, --file string   path to file to generate usage information for
  -h, --help          help for usage
  -o, --out string    path to output file
```

## OpenAPI Support

The tables below give an overview of what we support from OpenAPI.

### ✅ Server Configuration 

| Name | Notes | Swagger Link |Support |
|------|-------|--------------|--------|
| ServerURL config | Global and per-method (include base URL and templating) | [api-host-and-base-path](https://swagger.io/docs/specification/api-host-and-base-path/) |✅ |
| Authentication and security | Global and per-method | [authentication](https://swagger.io/docs/specification/authentication/) |✅ |
| Method generation |  |   |✅ |
| Model generation |  Request and response |   |✅ |

### ⚠️ Path Parameters Serialization ([path-parameters](https://swagger.io/docs/specification/describing-parameters/#path-parameters))

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Default | `(style = simple, explode = false)` | [serialization/#path](https://swagger.io/docs/specification/serialization/#path/) | ✅ |
| Basic types | | | ✅ |
| Simple objects | | | ✅ |
| `label` | | | ❌ |
| `matrix` | | | ❌ |

### ⚠️ Query Parameters Serialization ([query-parameters](https://swagger.io/docs/specification/describing-parameters/#query-parameters) & [query](https://swagger.io/docs/specification/describing-parameters/#query))

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| `json` | | | ✅ |
| `form` | | | ✅ |
| `spaceDelimited` | | | ✅ |
| `pipeDelimited` | | | ✅ |
| `deepObject` | | | ✅ |
| Basic types | | | ✅ |
| Simple objects | | | ✅ |

### ✅  Request Headers ([header](https://swagger.io/docs/specification/serialization/#header))

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Simple (explode = true)| | | ✅ |
| Simple (explode = false)| | | ✅ |

### ⚠️  Request Body Serialization

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Multipart encoding | | [multi-part requests](https://swagger.io/docs/specification/describing-request-body/multipart-requests/)| ✅ |
| Binary| | | ✅ |
| Form data | | | ✅ |
| JSON | Both `application/json` and `text/json`| | ✅ |
| `x-www-form-urlencoded` |  Including encoding, but not non-object types | [describing-request-body](https://swagger.io/docs/specification/describing-request-body)| ⚠️ |
| Plain text | | | ✅ |
| Raw byte | | | ✅ |
| Handling `required` body | | | ✅ |
| XML | | | ❌ |
| Other media types| | | ❌ |

### ⚠️  Response Body Deserialization

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Return StatusCode and Content-Type | | | ✅ |
| JSON | | | ✅ |
| Plain text | | | ✅ |
| Raw byte | | | ✅ |
| JSON | | | ✅ |
| Other media types| | | ❌ |

### ✅ Media-Type Patterns ([media-types](https://swagger.io/docs/specification/media-types))

### ✅ Datatypes

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Basic types | | [data-models/data-types](https://swagger.io/docs/specification/data-models/data-types/)| ✅ |
| Enums | | | ✅ |
| Number formats | float, double, int32, int64 | | ✅ |
| Date-time | | | ✅ |
| Binary | | | ✅ |
| Arrays | | | ✅ |
| Objects | | | ✅ |
| Optional | | | ✅ |
| Maps | | | ✅ |
| Any type | | | ✅ |
| OneOf/AnyOf/AllOf | anyOf treated as oneOf |[oneof-anyof-allof-not](https://swagger.io/docs/specification/data-models/oneof-anyof-allof-not/)| ✅ |

### ✅  Miscellaneous

| Name | Notes | Swagger Link | Support |
|------|-------|--------------|---------|
| Auxiliary files* | | | ✅ |
| `x-speakeasy-server-id` generation | | | ✅ |
| Snippet generation | | | ✅ |
| README generation | | | ✅ |
| Documentation generation | | | ✅ |

\* Utility classes and functions to help with serialization and deserialization.

\* Files needed for creating a fully compilable package that can be published to the relevant package manager without further changes.


## Advanced Generation Features

* [SDK Gen Configuration](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/customize-sdks/intro/) - Learn how to configure the SDK generator to your needs.
* [Generated Comments](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/sdk-docs/#comments) - Learn how comments are generated from your OpenAPI document and how to customize them.
* [Configuring the SDK with Server URLs](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/customize-sdks/servers/) - Learn how to configure the SDK to use different server URLs for different environments.
* [Readme Generation](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/sdk-docs/) - Learn how the SDK generates README.md files and how to control this.
* [Using Custom HTTP Clients With the SDK](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/customize-sdks/custom-http-client/) - Learn how to provide a custom HTTP Client to the SDKs at runtime.
* [Capturing Telemetry on SDK Usage](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/sdk-telemetry/) - Learn how you can capture telemetry to get an understanding of how your SDKs are being used.
* [Automated SDK Generation](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/github-setup/) - Use our GitHub Action and Workflows to set up CI/CD for generating and publishing your SDKs.
* [Override Generated Names](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/customize-sdks/methods/) - Speakeasy uses your OpenAPI schema to infer names for class types, methods, and parameters. However, you can override these names to tailor the generated SDK to your preferences.
* [Add Retries to Your SDKs](https://speakeasyapi.dev/docs/using-speakeasy/create-client-sdks/customize-sdks/retries/) - The generator supports the ability to generate SDKs that will automatically retry requests that fail due to network errors or any configured HTTP status code.

## Getting Support

If you need support using the Speakeasy CLI, please contact us via [email](info@speakeasyapi.dev), on [Slack](https://join.slack.com/t/speakeasy-dev/shared_invite/zt-1df0lalk5-HCAlpcQiqPw8vGukQWhexw), or file a GitHub issue and we'll respond ASAP!

<!-- WARNING: The below content is replaced by running `go run cmd/docs/main.go` please don't manually edit anything below this line -->

### Usage

```bash
speakeasy [flags]
```

#### Options

```bash
  -h, --help   help for speakeasy
```

#### Sub Commands

* [speakeasy api](docs/api/README.md)  - Access the Speakeasy API via the CLI
* [speakeasy auth](docs/auth/README.md)  - Authenticate the CLI
* [speakeasy generate](docs/generate/README.md)  - Generate Client SDKs and Terraform Providers
* [speakeasy validate](docs/validate/README.md)  - Validate OpenAPI documents

## CLI  
`speakeasy`  


The speakeasy cli tool provides access to the speakeasyapi.dev toolchain  

### Details

 A cli tool for interacting with the Speakeasy https://www.speakeasyapi.dev/ platform and its various functions including:
	- Generating Client SDKs from OpenAPI specs (go, python, typescript, java, php, c#, swift, ruby, terraform)
	- Validating OpenAPI specs
	- Interacting with the Speakeasy API to create and manage your API workspaces
	- Generating OpenAPI specs from your API traffic
	- Generating Postman collections from OpenAPI Specs


### Usage

```
speakeasy [flags]
```

#### Options

```
  -h, --help              help for speakeasy
      --logLevel string   the log level (available options: [info, warn, error]) (default "info")
```

#### Sub Commands

* [speakeasy ask](docs/ask.md)	 - Starts a conversation with Speakeasy trained AI
* [speakeasy auth](docs/auth/README.md)	 - Authenticate the CLI
* [speakeasy bump](docs/bump.md)	 - Bumps the version of a Speakeasy Generation Target
* [speakeasy configure](docs/configure/README.md)	 - Configure your Speakeasy SDK Setup.
* [speakeasy generate](docs/generate/README.md)	 - One off Generations for client SDKs and more
* [speakeasy lint](docs/lint/README.md)	 - Lint/Validate OpenAPI documents and Speakeasy configuration files
* [speakeasy merge](docs/merge.md)	 - Merge multiple OpenAPI documents into a single document
* [speakeasy openapi](docs/openapi/README.md)	 - Utilities for working with OpenAPI documents
* [speakeasy overlay](docs/overlay/README.md)	 - Work with OpenAPI Overlays
* [speakeasy quickstart](docs/quickstart.md)	 - Guided setup to help you create a new SDK in minutes.
* [speakeasy run](docs/run.md)	 - generate an SDK, compile OpenAPI sources, and much more from a workflow.yaml file
* [speakeasy suggest](docs/suggest/README.md)	 - Automatically improve your OpenAPI document with an LLM
* [speakeasy tag](docs/tag/README.md)	 - Add tags to a given revision of your API. Specific to a registry namespace
* [speakeasy update](docs/update.md)	 - Update the Speakeasy CLI to the latest version
